/*
btoptim.c Version 0.1.0  Btree optimize database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* btoptim calculates the optimum number */
/* records per block at each level of the */
/* database */
/* Optimization is based on access time, */
/* not on disk space */
/* This subroutine may be used to create */
/* a database reorganization program */

#include <bthash.h>

void btoptim(totrcds,nodesz,levels,
   lvltot,lvlrem,lvlblks,blktot)
int totrcds;
int nodesz;     /* records per block */
int *levels;    /* total number of levels in the database */
int *lvltot;    /* total records per node at each level */
int *lvlrem;    /* total blocks @ records+1 */
int *lvlblks;   /* total blocks per level */
int *blktot;    /* accumulated blocks at each level */
   {
   int lvl;
   int tot;
   int tmp;
   int subnodes;
   int nodes;
   int blks;
   *levels  = 0;
   *lvltot  = 0;
   *lvlrem  = 0;
   *lvlblks = 0;
   *blktot  = 0;
   if (!(nodesz & 1))
      {
      fprintf(stderr,"btoptim: nodesz "
	 "%d is not odd\n",
	 nodesz);
      return;
      } /* odd err */
   *(lvltot++)  = totrcds;
   *(lvlrem++)  = 0;
   *(lvlblks++) = 1;
   *(blktot++)  = 1;
   /* total left pointers + 1 right pointer */
   /* the number of blocks underneath a parent block */
   /* if nodesz = 3, there are 4 pointers */
   subnodes = nodesz + 1;
   tot    = totrcds;
   nodes  = subnodes;
   blks   = 1;
   lvl    = 1;
   while (tot/nodesz > 0)
      {
      tmp = tot / subnodes;
      if (tot % subnodes) tmp++;
      blks += nodes;
      *(lvltot++)  = tmp;
      *(lvlrem++)  = tot % subnodes;
      *(lvlblks++) = nodes;
      *(blktot++)  = blks;
      tot = tmp;
      nodes *= subnodes;
      lvl++;
      } /* for each level */
   *levels = lvl;
   } /* btoptim */
